@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -39,9 +39,7 @@
 #include "vfs/vfs.h"
 #include "vfs/vfsdirectory.h"
 #include "vfs/directoryprovider.h"
-#ifdef HAVE_ZIP
 #include "vfs/zip/zipprovider.h"
-#endif
 #include "eventchannel/eventmanager.h"
 #include "video/imagemanager.h"
 #include "video/animationmanager.h"
@@ -144,7 +142,8 @@ namespace FIFE {
 
 		m_imagemanager->invalidateAll();
 
-		m_renderbackend->setScreenMode(mode);
+		// recreate main screen
+		m_renderbackend->createMainScreen(mode,	m_settings.getWindowTitle(), m_settings.getWindowIcon());
 
 		if (m_guimanager) {
 			m_guimanager->resizeTopContainer(0,0,mode.getWidth(), mode.getHeight());
@@ -171,10 +170,10 @@ namespace FIFE {
 		FL_LOG(_log, "Adding root directory to VFS");
 		m_vfs->addSource( new VFSDirectory(m_vfs) );
 		m_vfs->addProvider( new DirectoryProvider() );
-#ifdef HAVE_ZIP
+
 		FL_LOG(_log, "Adding zip provider to VFS");
 		m_vfs->addProvider( new ZipProvider() );
-#endif
+
 		//m_vfs->addProvider(ProviderDAT2());
 		//m_vfs->addProvider(ProviderDAT1());
 		FL_LOG(_log, "Engine pre-init done");
@@ -190,6 +189,7 @@ namespace FIFE {
 		m_eventmanager = new EventManager();
 		m_eventmanager->setMouseSensitivity(m_settings.getMouseSensitivity());
 		m_eventmanager->setMouseAccelerationEnabled(m_settings.isMouseAccelerationEnabled());
+		m_eventmanager->setJoystickSupport(m_settings.isJoystickSupport());
 
 		FL_LOG(_log, "Creating resource managers");
 
@@ -305,9 +305,11 @@ namespace FIFE {
 		m_model->adoptPather(new RoutePather());
 		FL_LOG(_log, "Adding grid prototypes to model");
 		m_model->adoptCellGrid(new SquareGrid());
-		m_model->adoptCellGrid(new HexGrid());
+		m_model->adoptCellGrid(new HexGrid(false));
+		m_model->adoptCellGrid(new HexGrid(true));
 
 		m_cursor = new Cursor(m_renderbackend);
+		m_cursor->setNativeImageCursorEnabled(m_settings.isNativeImageCursorEnabled());
 		FL_LOG(_log, "Engine initialized");
 	}
 
@@ -362,10 +364,11 @@ namespace FIFE {
 		m_renderbackend->startFrame();
 		m_eventmanager->processEvents();
 		m_timemanager->update();
+		m_soundmanager->update();
 
-        m_renderbackend->clearBackBuffer();
 		m_targetrenderer->render();
 		if (m_model->getActiveCameraCount() == 0) {
+			m_renderbackend->clearBackBuffer();
 			m_offrenderer->render();
 		} else {
 			m_model->update();
@@ -397,6 +400,4 @@ namespace FIFE {
 			++i;
 		}
 	}
-}//FIFE
-
-/* vim: set noexpandtab: set shiftwidth=2: set tabstop=2: */
+}
