@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -22,7 +22,7 @@
 // Standard C++ library includes
 
 // 3rd party library includes
-#include <cegui-0/CEGUI/CEGUI.h>
+#include <CEGUI/CEGUI.h>
 
 // FIFE includes
 // These includes are split up in two parts, separated by one empty line
@@ -149,9 +149,11 @@ namespace FIFE {
 				// mousewheel up or down
 				int32_t wheelChange = event.wheel.y;
 				if (wheelChange != 0) {
-					/*if (event.wheel.direction == SDL_MOUSEWHEEL_FLIPPED) {
+#if SDL_VERSION_ATLEAST(2,0,4)
+					if (event.wheel.direction == SDL_MOUSEWHEEL_FLIPPED) {
 						wheelChange *= -1;
-					}*/
+					}
+#endif
 					consumed = CEGUI::System::getSingleton().getDefaultGUIContext().injectMouseWheelChange(wheelChange);
 				}
 				break;
