@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -44,9 +44,9 @@
 #include "video/fonts/fontbase.h"
 #include "video/fonts/truetypefont.h"
 #include "video/fonts/subimagefont.h"
-#include "eventchannel/key/ec_key.h"
-#include "eventchannel/key/ec_keyevent.h"
-#include "eventchannel/mouse/ec_mouseevent.h"
+#include "eventchannel/key/key.h"
+#include "eventchannel/key/keyevent.h"
+#include "eventchannel/mouse/mouseevent.h"
 #include "vfs/fife_boost_filesystem.h"
 
 #include "fifechanmanager.h"
@@ -61,9 +61,12 @@ namespace FIFE {
 		m_imgloader(new GuiImageLoader()) ,
 		m_input(new fcn::SDLInput()),
 		m_console(0),
+		m_cursor(0),
 		m_defaultfont(0),
 		m_fonts(),
-		m_logic_executed(false) {
+		m_logic_executed(false),
+		m_enabled_console(true),
+		m_backend("") {
 
 		m_fcn_gui->setInput(m_input);
 		fcn::Image::setImageLoader(m_imgloader);
@@ -167,9 +170,16 @@ namespace FIFE {
 	}
 
 	void FifechanManager::resizeTopContainer(uint32_t x, uint32_t y, uint32_t width, uint32_t height) {
+		if (m_backend == "SDL") {
+			static_cast<SdlGuiGraphics*>(m_gui_graphics)->updateTarget();
+		} else {
+			static_cast<OpenGLGuiGraphics*>(m_gui_graphics)->updateTarget();
+		}
 		m_fcn_topcontainer->setDimension(fcn::Rectangle(x, y, width, height));
-		this->invalidateFonts();
-		this->m_console->reLayout();
+		invalidateFonts();
+		if (m_console) {
+			m_console->reLayout();
+		}
 	}
 
 	fcn::Gui* FifechanManager::getFifechanGUI() const {
@@ -190,6 +200,14 @@ namespace FIFE {
 		}
 	}
 
+	void FifechanManager::setConsoleEnabled(bool console) {
+		m_enabled_console = console;
+	}
+
+	bool FifechanManager::isConsoleEnabled() const {
+		return m_enabled_console;
+	}
+
 	void FifechanManager::init(const std::string& backend, int32_t screenWidth, int32_t screenHeight) {
 		if( backend == "SDL" ) {
 			m_gui_graphics = new SdlGuiGraphics();
@@ -203,9 +221,12 @@ namespace FIFE {
 			//should never get here
 			assert(0);
 		}
+		m_backend = backend;
 
 		m_fcn_gui->setGraphics(m_gui_graphics);
-		m_console = new Console();
+		if (m_enabled_console) {
+			m_console = new Console();
+		}
 
 		resizeTopContainer(0, 0, screenWidth, screenHeight);
 	}
@@ -369,6 +390,13 @@ namespace FIFE {
 		return mouseevt;
 	}
 
+	void FifechanManager::setTabbingEnabled(bool tabbing) {
+		m_fcn_gui->setTabbingEnabled(tabbing);
+	}
+	
+	bool FifechanManager::isTabbingEnabled() const {
+		return m_fcn_gui->isTabbingEnabled();
+	}
 
 	int32_t FifechanManager::convertFifechanKeyToFifeKey(int32_t value) {
 
