@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -153,8 +153,11 @@ namespace FIFE {
 	
 	bool LibRocketInputProcessor::processMouseWheelMotion(SDL_Event& event) {
 		// mousewheel up
-		//if (event.wheel.y > 0 || (event.wheel.direction == SDL_MOUSEWHEEL_FLIPPED && event.wheel.y < 0)) {
+#if SDL_VERSION_ATLEAST(2,0,4)
+		if (event.wheel.y > 0 || (event.wheel.direction == SDL_MOUSEWHEEL_FLIPPED && event.wheel.y < 0)) {
+#else
 		if (event.wheel.y > 0) {
+#endif
 			if(m_wheelCounter <= 0) {
 				m_wheelCounter--;
 			} else {
@@ -164,8 +167,11 @@ namespace FIFE {
 			}
 		}
 		// mousewheel down
-		//else if (event.wheel.y < 0 || (event.wheel.direction == SDL_MOUSEWHEEL_FLIPPED && event.wheel.y > 0)) {
+#if SDL_VERSION_ATLEAST(2,0,4)
+		else if (event.wheel.y < 0 || (event.wheel.direction == SDL_MOUSEWHEEL_FLIPPED && event.wheel.y > 0)) {
+#else
 		else if (event.wheel.y < 0) {
+#endif
 			if(m_wheelCounter >= 0) {
 				m_wheelCounter++;
 			} else {
