@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -319,7 +319,6 @@ namespace FIFE {
 											int r = 0;
 											int stackpos = 0;
 											int cellStack = 0;
-											int visitorRadius = 0;
 
 											const std::string* instanceId = instance->Attribute(std::string("id"));
 											const std::string* objectId = instance->Attribute(std::string("o"));
@@ -364,16 +363,6 @@ namespace FIFE {
 
 											int stackRetVal = instance->QueryValueAttribute("stackpos", &stackpos);
 											int cellStackRetVal = instance->QueryValueAttribute("cellstack", &cellStack);
-											int visitorRetVal = instance->QueryValueAttribute("visitor_radius", &visitorRadius);
-											const std::string* shapeType = instance->Attribute(std::string("visitor_shape"));
-											VisitorShapeInfo visitorShape = ITYPE_NO_SHAPE;
-											if (shapeType) {
-												if ("quad" == *shapeType) {
-													visitorShape = ITYPE_QUAD_SHAPE;
-												} else if ("circle" == *shapeType) {
-													visitorShape = ITYPE_CIRCLE_SHAPE;
-												}
-											}
 
 											if (objectId) {
 												if (namespaceId) {
@@ -413,12 +402,6 @@ namespace FIFE {
 															inst->setCellStackPosition(cellStack);
 														}
 
-														if  (visitorRetVal == TIXML_SUCCESS) {
-															inst->setVisitor(true);
-															inst->setVisitorRadius(visitorRadius);
-															inst->setVisitorShape(visitorShape);
-														}
-
 														if (costId) {
 															double cost = 0;
 															int costRetVal = instance->QueryValueAttribute("cost", &cost);
@@ -490,17 +473,6 @@ namespace FIFE {
 												ModelCoordinate mc(cellX, cellY);
 												Cell* cell = cache->createCell(mc);
 
-												const std::string* cellVisual = cellElement->Attribute(std::string("state"));
-												if (cellVisual) {
-													CellVisualEffect cve = CELLV_REVEALED;
-													if (*cellVisual == "concealed") {
-														cve = CELLV_CONCEALED;
-													} else if (*cellVisual == "masked") {
-														cve = CELLV_MASKED;
-													}
-													cell->setFoWType(cve);
-												}
-
 												const std::string* cellBlocker = cellElement->Attribute(std::string("blocker_type"));
 												if (cellBlocker) {
 													if (*cellBlocker == "no_blocker") {
@@ -675,14 +647,13 @@ namespace FIFE {
 
 					for (const TiXmlElement* cameraElement = root->FirstChildElement("camera"); cameraElement; cameraElement = cameraElement->NextSiblingElement("camera")) {
 						const std::string* cameraId = cameraElement->Attribute(std::string("id"));
-						const std::string* refLayerId = cameraElement->Attribute(std::string("ref_layer_id"));
 
 						int refCellWidth = 0;
 						int refCellHeight = 0;
 						int success = cameraElement->QueryIntAttribute("ref_cell_width", &refCellWidth);
 						success &= cameraElement->QueryIntAttribute("ref_cell_height", &refCellHeight);
 
-						if (cameraId && refLayerId && success == TIXML_SUCCESS) {
+						if (cameraId && success == TIXML_SUCCESS) {
 							double tilt = 0.0;
 							double zoom = 1.0;
 							double rotation = 0.0;
@@ -694,48 +665,38 @@ namespace FIFE {
 
 							const std::string* viewport = cameraElement->Attribute(std::string("viewport"));
 
-							Layer* layer = NULL;
-							try {
-								layer = map->getLayer(*refLayerId);
-							}
-							catch (NotFound&) {
-								// TODO - handle exception
-								assert(false);
-							}
-
 							Camera* cam = NULL;
-							if (layer) {
-								if (viewport) {
-									// parse out the viewport parameters
-									IntVector viewportParameters = tokenize(*viewport, ',');
 
-									// make sure the right number of viewport parameters were parsed
-									if (viewportParameters.size() == 4) {
-										Rect rect(viewportParameters[0], viewportParameters[1],
-													viewportParameters[2], viewportParameters[3]);
+							if (viewport) {
+								// parse out the viewport parameters
+								IntVector viewportParameters = tokenize(*viewport, ',');
 
-										try {
-											cam = map->addCamera(*cameraId, layer, rect);
-										}
-										catch (NameClash&) {
-											// TODO - handle exception
-											assert(false);
-										}
-									}
-								}
-								else {
-									Rect rect(0, 0, m_renderBackend->getScreenWidth(), m_renderBackend->getScreenHeight());
+								// make sure the right number of viewport parameters were parsed
+								if (viewportParameters.size() == 4) {
+									Rect rect(viewportParameters[0], viewportParameters[1],
+												viewportParameters[2], viewportParameters[3]);
 
 									try {
-										cam = map->addCamera(*cameraId, layer, rect);
+										cam = map->addCamera(*cameraId, rect);
 									}
 									catch (NameClash&) {
 										// TODO - handle exception
 										assert(false);
 									}
 								}
+							} else {
+								Rect rect(0, 0, m_renderBackend->getScreenWidth(), m_renderBackend->getScreenHeight());
+
+								try {
+									cam = map->addCamera(*cameraId, rect);
+								}
+								catch (NameClash&) {
+									// TODO - handle exception
+									assert(false);
+								}
 							}
 
+
 							if (cam) {
 								cam->setCellImageDimensions(refCellWidth, refCellHeight);
 								cam->setRotation(rotation);
