@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -37,7 +37,7 @@
 namespace FIFE {
 	class HexGrid: public CellGrid {
 	public:
-		HexGrid();
+		HexGrid(bool axial = false);
 		virtual ~HexGrid();
 
 		bool isAccessible(const ModelCoordinate& curpos, const ModelCoordinate& target);
@@ -58,6 +58,8 @@ namespace FIFE {
 	private:
 		double getXZigzagOffset(double y);
 		ModelCoordinate toLayerCoordinatesHelper(const ExactModelCoordinate& coords);
+
+		bool m_axial;
 	};
 }
 
