@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -245,16 +245,6 @@ namespace FIFE {
 			 */
 			int32_t getMaxNeighborZ();
 
-			/** Sets whether the CellCache need to be updated.
-			 * @param updated A boolean, true means no update is needed, false indicates a update.
-			 */
-			void setUpdated(bool updated);
-
-			/** Gets whether the CellCache need to be updated.
-			 * @return A boolean, true means no update is needed, false indicates a update.
-			 */
-			bool isUpdated();
-
 			/** Returns all cells in the line.
 			 * @param pt1 A const reference to the ModelCoordinate where the line begin.
 			 * @param pt2 A const reference to the ModelCoordinate where the line end.
@@ -269,6 +259,12 @@ namespace FIFE {
 			 */
 			std::vector<Cell*> getCellsInRect(const Rect& rec);
 
+			/** Returns all blocking cells in the rect.
+			* @param rec A const reference to the Rect which specifies the size.
+			* @return A vector that contain the cells.
+			*/
+			std::vector<Cell*> getBlockingCellsInRect(const Rect& rec);
+
 			/** Returns all cells in the circle.
 			 * @param center A const reference to the ModelCoordinate where the center of the circle is.
 			 * @param radius A unsigned integer, radius of the circle.
@@ -608,7 +604,6 @@ namespace FIFE {
 			bool isStaticSize();
 
 			void setBlockingUpdate(bool update);
-			void setFowUpdate(bool update);
 			void setSizeUpdate(bool update);
 			void update();
 		private:
@@ -652,15 +647,9 @@ namespace FIFE {
 			//! indicates blocking update
 			bool m_blockingUpdate;
 
-			//! indicates fow update
-			bool m_fowUpdate;
-
 			//! indicates size update
 			bool m_sizeUpdate;
 
-			//! need update
-			bool m_updated;
-
 			//! is automatic seach enabled
 			bool m_searchNarrow;
 
