@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -23,12 +23,13 @@
 #include <iostream>
 
 // 3rd party library includes
-#include <SDL.h>
+//#include <SDL.h>
 
 // FIFE includes
 // These includes are split up in two parts, separated by one empty line
 // First block: files included from the FIFE root src directory
 // Second block: files included from the same folder
+#include "audio/soundsource.h"
 #include "util/log/logger.h"
 #include "util/base/exception.h"
 #include "util/math/fife_math.h"
@@ -114,7 +115,8 @@ namespace FIFE {
 		m_oldLocation(source.m_location),
 		m_rotation(source.m_rotation),
 		m_oldRotation(source.m_rotation),
-		m_action(),
+		m_action(NULL),
+		m_soundSource(NULL),
 		m_speed(0),
 		m_timeMultiplier(1.0),
 		m_sayText(""),
@@ -131,6 +133,7 @@ namespace FIFE {
 		delete m_actionInfo;
 		delete m_sayInfo;
 		delete m_timeProvider;
+		delete m_soundSource;
 	}
 
 	void Instance::InstanceActivity::update(Instance& source) {
@@ -202,9 +205,6 @@ namespace FIFE {
 		m_visual(NULL),
 		m_blocking(object->isBlocking()),
 		m_overrideBlocking(false),
-		m_isVisitor(false),
-		m_visitorShape(ITYPE_CIRCLE_SHAPE),
-		m_visitorRadius(0),
 		m_cellStackPos(object->getCellStackPosition()),
 		m_specialCost(object->isSpecialCost()),
 		m_cost(object->getCost()),
@@ -431,6 +431,16 @@ namespace FIFE {
 		if (m_activity->m_actionInfo->m_action != old_action) {
 			m_activity->m_actionInfo->m_action_start_time = m_activity->m_actionInfo->m_prev_call_time;
 	    }
+		// start sound
+		if (m_activity->m_actionInfo->m_action->getAudio()) {
+			if (!m_activity->m_soundSource) {
+				m_activity->m_soundSource = new SoundSource(this);
+			}
+			m_activity->m_soundSource->setActionAudio(m_activity->m_actionInfo->m_action->getAudio());
+		} else if (old_action && old_action->getAudio()) {
+			m_activity->m_soundSource->setActionAudio(NULL);
+		}
+
 		if (isMultiObject()) {
 			std::vector<Instance*>::iterator multi_it = m_multiInstances.begin();
 			for (; multi_it != m_multiInstances.end(); ++multi_it) {
@@ -528,30 +538,6 @@ namespace FIFE {
 		return NULL;
 	}
 
-	void Instance::setVisitor(bool visit) {
-		m_isVisitor = visit;
-	}
-
-	bool Instance::isVisitor() {
-		return m_isVisitor;
-	}
-
-	void Instance::setVisitorShape(VisitorShapeInfo info) {
-		m_visitorShape = info;
-	}
-
-	VisitorShapeInfo Instance::getVisitorShape() {
-		return m_visitorShape;
-	}
-
-	void Instance::setVisitorRadius(uint16_t radius) {
-		m_visitorRadius = radius;
-	}
-
-	uint16_t Instance::getVisitorRadius() {
-		return m_visitorRadius;
-	}
-
 	void Instance::setCellStackPosition(uint8_t stack) {
 		m_cellStackPos = stack;
 	}
@@ -813,6 +799,11 @@ namespace FIFE {
 		// it is the same action as the finalized action
 		m_activity->m_action = NULL;
 
+		// stop audio
+		if (action->getAudio() && m_activity->m_soundSource) {
+			m_activity->m_soundSource->setActionAudio(NULL);
+		}
+
 		if (isMultiObject()) {
 			std::vector<Instance*>::iterator multi_it = m_multiInstances.begin();
 			for (; multi_it != m_multiInstances.end(); ++multi_it) {
