@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -47,6 +47,7 @@ namespace FIFE {
 	class Instance;
 	class ActionInfo;
 	class SayInfo;
+	class SoundSource;
 	class TimeProvider;
 	class Route;
 
@@ -87,13 +88,6 @@ namespace FIFE {
 		virtual void onInstanceDeleted(Instance* instance) =0;
 	};
 
-	enum VisitorShapeType {
-		ITYPE_NO_SHAPE = 0,
-		ITYPE_QUAD_SHAPE,
-		ITYPE_CIRCLE_SHAPE
-	};
-	typedef uint8_t VisitorShapeInfo;
-
 	/** An Instance is an "instantiation" of an Object at a Location.
 	 *
 	 */
@@ -385,30 +379,6 @@ namespace FIFE {
 		 */
 		Route* getRoute();
 
-		/** Marks this instance as a visitor.
-		 */
-		void setVisitor(bool visit);
-
-		/** If instance is a visitor it returns true otherwise false.
-		 */
-		bool isVisitor();
-
-		/** Sets the shape type for a visitor.
-		*/
-		void setVisitorShape(VisitorShapeInfo info);
-
-		/** Gets the shape type for a visitor.
-		*/
-		VisitorShapeInfo getVisitorShape();
-
-		/** Sets the range for a visitor.
-		*/
-		void setVisitorRadius(uint16_t radius);
-
-		/** Gets the visitor range.
-		*/
-		uint16_t getVisitorRadius();
-
 		/** Sets the cell stack position.
 		*/
 		void setCellStackPosition(uint8_t stack);
@@ -572,6 +542,8 @@ namespace FIFE {
 			int32_t m_oldRotation;
 			//! action on previous round. NOTE: might become invalid, only used for address comparison
 			Action* m_action;
+			//! sound source of action audio on previous round
+			SoundSource* m_soundSource;
 			//! speed on previous round
 			double m_speed;
 			//! time multiplier on previous round
@@ -613,12 +585,6 @@ namespace FIFE {
 		bool m_blocking;
 		//! allow to override the blocking property
 		bool m_overrideBlocking;
-		//! is instance a visitor (FoW)
-		bool m_isVisitor;
-		//! visitor shape type
-		VisitorShapeInfo m_visitorShape;
-		//! visitor radius (FoW)
-		uint16_t m_visitorRadius;
 		//! position on cell stack
 		uint8_t m_cellStackPos;
 		//! indicates special cost
