@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -241,7 +241,7 @@ namespace FIFE {
 
 		// With OpenGL 2.0 or GL_ARB_texture_non_power_of_two we don't really need to care
 		// about non power of 2 textures 
-		if(GLEE_ARB_texture_non_power_of_two && RenderBackend::instance()->isNPOTEnabled()) {
+		if(GLEW_ARB_texture_non_power_of_two && RenderBackend::instance()->isNPOTEnabled()) {
 			m_chunk_size_w = width;
 			m_chunk_size_h = height;
 		}
@@ -307,7 +307,7 @@ namespace FIFE {
 		}
 
 		GLint internalFormat = GL_RGBA8;
-		if(GLEE_ARB_texture_compression && RenderBackend::instance()->isImageCompressingEnabled()) {
+		if(GLEW_ARB_texture_compression && RenderBackend::instance()->isImageCompressingEnabled()) {
 			internalFormat = GL_COMPRESSED_RGBA;
 			m_compressed = true;
 		} else {
@@ -364,7 +364,7 @@ namespace FIFE {
 			return;
 		}
 		
-		if(GLEE_ARB_texture_non_power_of_two && RenderBackend::instance()->isNPOTEnabled()) {
+		if(GLEW_ARB_texture_non_power_of_two && RenderBackend::instance()->isNPOTEnabled()) {
 			if(RenderBackend::instance()->isColorKeyEnabled()) {
 				uint8_t* oglbuffer = new uint8_t[width * height * 4];
 				memcpy(oglbuffer, data, width * height * 4 * sizeof(uint8_t));
@@ -465,7 +465,7 @@ namespace FIFE {
 		uint32_t width = shared->getWidth();
 		uint32_t height = shared->getHeight();
 
-		if(!GLEE_ARB_texture_non_power_of_two || !RenderBackend::instance()->isNPOTEnabled()) {
+		if(!GLEW_ARB_texture_non_power_of_two || !RenderBackend::instance()->isNPOTEnabled()) {
 			width = nextPow2(width);
 			height = nextPow2(height);
 		}
@@ -484,7 +484,7 @@ namespace FIFE {
 		}
 	}
 
-	void GLImage::useSharedImage(const ImagePtr& shared, const Rect& region) {
+	void GLImage::useSharedImage(const ImagePtr& shared, const Rect& region){
 		GLImage* img = static_cast<GLImage*>(shared.get());
 
 		m_shared_img = img;
@@ -500,7 +500,7 @@ namespace FIFE {
 			generateGLSharedTexture(img, region);
 		}
 
-		setState(IResource::RES_LOADED);
+		setState(m_shared_img->getState());
 	}
 
 	void GLImage::forceLoadInternal() {
