@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -119,7 +119,7 @@ namespace FIFE {
 
 	RenderBackendOpenGL::~RenderBackendOpenGL() {
 		glDeleteTextures(1, &m_maskOverlay);
-		if(GLEE_EXT_framebuffer_object && m_useframebuffer) {
+		if(GLEW_EXT_framebuffer_object && m_useframebuffer) {
 			glDeleteFramebuffers(1, &m_fbo_id);
 		}
 		SDL_GL_DeleteContext(m_context);
@@ -142,9 +142,12 @@ namespace FIFE {
 				throw SDLException(SDL_GetError());
 			}
 		}
-		// defines buffer sizes
-		SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 2);
+
+		// setup OpenGL
+		SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
 		SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 0);
+
+		// defines buffer sizes
 		SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
 		SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);
 		SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
@@ -153,9 +156,6 @@ namespace FIFE {
 		SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
 		SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, 32);
 		//SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 32);
-
-		// sync swaping with refresh rate if VSync is enabled
-		SDL_GL_SetSwapInterval(static_cast<uint8_t>(m_vSync));
 	}
 
 	void RenderBackendOpenGL::clearBackBuffer() {
@@ -179,15 +179,20 @@ namespace FIFE {
 	}
 
 	void RenderBackendOpenGL::setScreenMode(const ScreenMode& mode) {
+		bool recreate = m_window != NULL;
 		uint16_t width = mode.getWidth();
 		uint16_t height = mode.getHeight();
 		uint16_t bitsPerPixel = mode.getBPP();
 		uint32_t flags = mode.getSDLFlags();
 		// in case of recreating
-		if (m_window) {
-			SDL_GL_DeleteContext(m_context);
+		if (recreate) {
 			SDL_DestroyWindow(m_window);
+			m_window = NULL;
 			m_screen = NULL;
+			
+			if (GLEW_EXT_framebuffer_object && m_useframebuffer) {
+				glDeleteFramebuffers(1, &m_fbo_id);
+			}
 		}
 		// create window
 		uint8_t displayIndex = mode.getDisplay();
@@ -210,8 +215,14 @@ namespace FIFE {
 			throw SDLException(SDL_GetError());
 		}
 
-		// create render context
-		m_context = SDL_GL_CreateContext(m_window);
+		// create render context or use the old with new window
+		if (recreate) {
+			if (SDL_GL_MakeCurrent(m_window, m_context) < 0) {
+				throw SDLException(SDL_GetError());
+			}
+		} else {
+			m_context = SDL_GL_CreateContext(m_window);
+		}
 		// set the window surface as main surface, not really needed anymore
 		m_screen = SDL_GetWindowSurface(m_window);
 		m_target = m_screen;
@@ -219,6 +230,13 @@ namespace FIFE {
 			throw SDLException(SDL_GetError());
 		}
 
+		// initialize GLEW
+		glewExperimental = GL_TRUE;
+		GLenum glewError = glewInit();
+		if (glewError != GLEW_OK) {
+			FL_LOG(_log, LMsg("RenderBackendOpenGL") << "Error initializing GLEW!" << glewGetErrorString(glewError));
+		}
+
 		FL_LOG(_log, LMsg("RenderBackendOpenGL")
 			<< "Videomode " << width << "x" << height
 			<< " at " << int32_t(bitsPerPixel) << " bpp with " << displayMode.refresh_rate << " Hz");
@@ -255,9 +273,12 @@ namespace FIFE {
 		glPixelStorei(GL_PACK_ALIGNMENT, 1);
 		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 
-		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
-		glClearDepth(1.0f);
-		glClearStencil(0);
+		// dont reset
+		if (!recreate) {
+			glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
+			glClearDepth(1.0f);
+			glClearStencil(0);
+		}
 
 		glEnable(GL_BLEND);
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -277,12 +298,12 @@ namespace FIFE {
 		glPointSize(1.0);
 		glLineWidth(1.0);
 
-		if(GLEE_EXT_framebuffer_object && m_useframebuffer) {
+		if(GLEW_EXT_framebuffer_object && m_useframebuffer) {
 			glGenFramebuffers(1, &m_fbo_id);
 		}
 		
 		if (m_textureFilter == TEXTURE_FILTER_ANISOTROPIC) {
-			if (GLEE_EXT_texture_filter_anisotropic) {
+			if (GLEW_EXT_texture_filter_anisotropic) {
 				GLint largest = 0;
 				glGetIntegerv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &largest);
 				m_maxAnisotropy = static_cast<int32_t>(largest);
@@ -293,8 +314,31 @@ namespace FIFE {
 			}
 		}
 
+		// sync swaping with refresh rate if VSync is enabled
+		SDL_GL_SetSwapInterval(static_cast<uint8_t>(m_vSync));
+
 		// currently unused, 1000 objects x 400 textures x 4 renderDataZ
 		//m_renderZ_datas.resize(1600000);
+
+		// 6 indices x 100000 objects
+		m_indices.resize(600000);
+		// a rough way to fill the index buffer, result is: 0, 1, 2, 0, 2, 3 | 4, 5, 6, 4, 6, 7
+		uint32_t index = 0;
+		for(std::vector<uint32_t>::size_type i = 0; i != m_indices.size(); i+=6) {
+			m_indices[i] = index;
+			m_indices[i+1] = index+1;
+			m_indices[i+2] = index+2;
+			m_indices[i+3] = index;
+			m_indices[i+4] = index+2;
+			m_indices[i+5] = index+3;
+			index += 4;
+		}
+
+		// currently unused, create buffer and send data
+		//glGenBuffers(1, &m_indicebufferId);
+		//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, m_indicebufferId);
+		//glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(uint32_t), &indices[0], GL_STATIC_DRAW);
+		//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
 	}
 
 	void RenderBackendOpenGL::startFrame() {
@@ -756,7 +800,7 @@ namespace FIFE {
 		bool render = false;
 
 		// render mode
-		GLenum mode = GL_QUADS;
+		GLenum mode = GL_TRIANGLES;
 		// texture id
 		uint32_t texture_id = 0;
 		uint32_t texture_id2 = 0;
@@ -783,6 +827,9 @@ namespace FIFE {
 		int32_t* currentIndex = 0;
 		uint32_t* currentElements = 0;
 
+		// index buffer pointer
+		uint32_t* indexBuffer = 0;
+
 		//stride
 		const uint32_t strideP = sizeof(renderDataP);
 		const uint32_t strideT = sizeof(renderDataT);
@@ -800,6 +847,7 @@ namespace FIFE {
 				disableColorArray();
 				setVertexPointer(2, strideT, &m_renderTextureDatas[0].vertex);
 				setTexCoordPointer(0, strideT, &m_renderTextureDatas[0].texel);
+				indexBuffer = &m_tIndices[0];
 				currentIndex = &indexT;
 				currentElements = &elementsT;
 			// texture with color/alpha
@@ -809,6 +857,7 @@ namespace FIFE {
 				setVertexPointer(2, strideTC, &m_renderTextureColorDatas[0].vertex);
 				setTexCoordPointer(0, strideTC, &m_renderTextureColorDatas[0].texel);
 				setColorPointer(strideTC, &m_renderTextureColorDatas[0].color);
+				indexBuffer = &m_tcIndices[0];
 				currentIndex = &indexTC;
 				currentElements = &elementsTC;
 			// primitive
@@ -817,6 +866,7 @@ namespace FIFE {
 				enableColorArray();
 				setVertexPointer(2, strideP, &m_renderPrimitiveDatas[0].vertex);
 				setColorPointer(strideP, &m_renderPrimitiveDatas[0].color);
+				indexBuffer = &m_pIndices[0];
 				currentIndex = &indexP;
 				currentElements = &elementsP;
 			}
@@ -826,6 +876,7 @@ namespace FIFE {
 			setVertexPointer(2, stride2TC, &m_renderMultitextureDatas[0].vertex);
 			setColorPointer(stride2TC, &m_renderMultitextureDatas[0].color);
 			setTexCoordPointer(0, stride2TC, &m_renderMultitextureDatas[0].texel);
+			indexBuffer = &m_tc2Indices[0];
 			currentIndex = &index2TC;
 			currentElements = &elements2TC;
 		}
@@ -837,6 +888,9 @@ namespace FIFE {
 			if (ro.mode != mode) {
 				type = true;
 				render = true;
+			} else if (ro.mode == GL_LINE_STRIP || ro.mode == GL_LINE_LOOP) {
+				// do not batch line strips, loops or triangle fans to avoid side effects
+				render = true;
 			}
 			if (ro.texture_id != texture_id) {
 				texture = true;
@@ -877,7 +931,7 @@ namespace FIFE {
 			if (render) {
 				if (*currentElements > 0) {
 					//render
-					glDrawArrays(mode, *currentIndex, *currentElements);
+					glDrawElements(mode, *currentElements, GL_UNSIGNED_INT, indexBuffer + *currentIndex);
 					*currentIndex += *currentElements;
 				}
 				// switch mode
@@ -894,11 +948,13 @@ namespace FIFE {
 								setVertexPointer(2, strideTC, &m_renderTextureColorDatas[0].vertex);
 								setTexCoordPointer(0, strideTC, &m_renderTextureColorDatas[0].texel);
 								setColorPointer(strideTC, &m_renderTextureColorDatas[0].color);
+								indexBuffer = &m_tcIndices[0];
 								currentElements = &elementsTC;
 								currentIndex = &indexTC;
 							} else {
 								setVertexPointer(2, strideP, &m_renderPrimitiveDatas[0].vertex);
 								setColorPointer(strideP, &m_renderPrimitiveDatas[0].color);
+								indexBuffer = &m_pIndices[0];
 								currentElements = &elementsP;
 								currentIndex = &indexP;
 							}
@@ -908,6 +964,7 @@ namespace FIFE {
 						if (ro.overlay_type == OVERLAY_TYPE_NONE) {
 							setVertexPointer(2, strideT, &m_renderTextureDatas[0].vertex);
 							setTexCoordPointer(0, strideT, &m_renderTextureDatas[0].texel);
+							indexBuffer = &m_tIndices[0];
 							currentElements = &elementsT;
 							currentIndex = &indexT;
 						}
@@ -928,17 +985,20 @@ namespace FIFE {
 								setVertexPointer(2, strideTC, &m_renderTextureColorDatas[0].vertex);
 								setTexCoordPointer(0, strideTC, &m_renderTextureColorDatas[0].texel);
 								setColorPointer(strideTC, &m_renderTextureColorDatas[0].color);
+								indexBuffer = &m_tcIndices[0];
 								currentElements = &elementsTC;
 								currentIndex = &indexTC;
 							} else {
 								setVertexPointer(2, strideT, &m_renderTextureDatas[0].vertex);
 								setTexCoordPointer(0, strideT, &m_renderTextureDatas[0].texel);
+								indexBuffer = &m_tIndices[0];
 								currentIndex = &indexT;
 								currentElements = &elementsT;
 							}
 						} else {
 							setVertexPointer(2, strideP, &m_renderPrimitiveDatas[0].vertex);
 							setColorPointer(strideP, &m_renderPrimitiveDatas[0].color);
+							indexBuffer = &m_pIndices[0];
 							currentElements = &elementsP;
 							currentIndex = &indexP;
 						}
@@ -956,6 +1016,7 @@ namespace FIFE {
 						setColorPointer(stride2TC, &m_renderMultitextureDatas[0].color);
 						setTexCoordPointer(1, stride2TC, &m_renderMultitextureDatas[0].texel2);
 						setTexCoordPointer(0, stride2TC, &m_renderMultitextureDatas[0].texel);
+						indexBuffer = &m_tc2Indices[0];
 
 						texture_id2 = m_maskOverlay;
 						currentElements = &elements2TC;
@@ -973,6 +1034,7 @@ namespace FIFE {
 						setColorPointer(stride2TC, &m_renderMultitextureDatas[0].color);
 						setTexCoordPointer(2, stride2TC, &m_renderMultitextureDatas[0].texel2);
 						setTexCoordPointer(0, stride2TC, &m_renderMultitextureDatas[0].texel);
+						indexBuffer = &m_tc2Indices[0];
 
 						texture_id2 = ro.overlay_id;
 						currentElements = &elements2TC;
@@ -990,6 +1052,7 @@ namespace FIFE {
 						setColorPointer(stride2TC, &m_renderMultitextureDatas[0].color);
 						setTexCoordPointer(3, stride2TC, &m_renderMultitextureDatas[0].texel2);
 						setTexCoordPointer(0, stride2TC, &m_renderMultitextureDatas[0].texel);
+						indexBuffer = &m_tc2Indices[0];
 
 						texture_id2 = ro.overlay_id;
 						currentElements = &elements2TC;
@@ -1010,11 +1073,13 @@ namespace FIFE {
 								setVertexPointer(2, strideTC, &m_renderTextureColorDatas[0].vertex);
 								setTexCoordPointer(0, strideTC, &m_renderTextureColorDatas[0].texel);
 								setColorPointer(strideTC, &m_renderTextureColorDatas[0].color);
+								indexBuffer = &m_tcIndices[0];
 								currentElements = &elementsTC;
 								currentIndex = &indexTC;
 							} else {
 								setVertexPointer(2, strideT, &m_renderTextureDatas[0].vertex);
 								setTexCoordPointer(0, strideT, &m_renderTextureDatas[0].texel);
+								indexBuffer = &m_tIndices[0];
 								currentElements = &elementsT;
 								currentIndex = &indexT;
 							}
@@ -1025,6 +1090,7 @@ namespace FIFE {
 						if (ro.overlay_type == OVERLAY_TYPE_NONE) {
 							setVertexPointer(2, strideP, &m_renderPrimitiveDatas[0].vertex);
 							setColorPointer(strideP, &m_renderPrimitiveDatas[0].color);
+							indexBuffer = &m_pIndices[0];
 							currentElements = &elementsP;
 							currentIndex = &indexP;
 						}
@@ -1073,7 +1139,7 @@ namespace FIFE {
 			}
 		}
 		// render
-		glDrawArrays(mode, *currentIndex, *currentElements);
+		glDrawElements(mode, *currentElements, GL_UNSIGNED_INT, indexBuffer + *currentIndex);
 
 		// reset all states
 		if (overlay_type != OVERLAY_TYPE_NONE) {
@@ -1096,6 +1162,11 @@ namespace FIFE {
 		m_renderTextureColorDatas.clear();
 		m_renderMultitextureDatas.clear();
 		m_renderObjects.clear();
+
+		m_pIndices.clear();
+		m_tIndices.clear();
+		m_tcIndices.clear();
+		m_tc2Indices.clear();
 	}
 
 	void RenderBackendOpenGL::renderWithZ() {
@@ -1128,7 +1199,7 @@ namespace FIFE {
 			if (ro.texture_id != texture_id) {
 				if (*currentElements > 0) {
 					//render
-					glDrawArrays(GL_QUADS, *currentIndex, *currentElements);
+					glDrawElements(GL_TRIANGLES, *currentElements, GL_UNSIGNED_INT, &m_indices[*currentIndex]);
 					*currentIndex += *currentElements;
 				}
 
@@ -1141,15 +1212,15 @@ namespace FIFE {
 				}
 
 				// set element to current size
-				*currentElements = 4;
+				*currentElements = 6;
 			} else {
 				// else add the element
-				*currentElements += 4;
+				*currentElements += 6;
 			}
 		}
 
 		// render
-		glDrawArrays(GL_QUADS, *currentIndex, *currentElements);
+		glDrawElements(GL_TRIANGLES, *currentElements, GL_UNSIGNED_INT, &m_indices[*currentIndex]);
 
 		//reset all states
 		disableLighting();
@@ -1222,7 +1293,7 @@ namespace FIFE {
 			if (ro.texture_id != texture_id) {
 				if (*currentElements > 0) {
 					//render
-					glDrawArrays(GL_QUADS, *currentIndex, *currentElements);
+					glDrawElements(GL_TRIANGLES, *currentElements, GL_UNSIGNED_INT, &m_indices[*currentIndex]);
 					*currentIndex += *currentElements;
 				}
 
@@ -1235,15 +1306,15 @@ namespace FIFE {
 				}
 
 				// set element to current size
-				*currentElements = 4;
+				*currentElements = 6;
 			} else {
 				// else add the element
-				*currentElements += 4;
+				*currentElements += 6;
 			}
 		}
 
 		// render
-		glDrawArrays(GL_QUADS, *currentIndex, *currentElements);
+		glDrawElements(GL_TRIANGLES, *currentElements, GL_UNSIGNED_INT, &m_indices[*currentIndex]);
 
 		//reset all states
 		disableLighting();
@@ -1311,7 +1382,7 @@ namespace FIFE {
 			if (render) {
 				if (*currentElements > 0) {
 					//render
-					glDrawArrays(GL_QUADS, *currentIndex, *currentElements);
+					glDrawElements(GL_TRIANGLES, *currentElements, GL_UNSIGNED_INT, &m_indices[*currentIndex]);
 					*currentIndex += *currentElements;
 				}
 				// multitexturing
@@ -1379,7 +1450,7 @@ namespace FIFE {
 			}
 		}
 		// render
-		glDrawArrays(GL_QUADS, *currentIndex, *currentElements);
+		glDrawElements(GL_TRIANGLES, *currentElements, GL_UNSIGNED_INT, &m_indices[*currentIndex]);
 
 		//reset all states
 		if (overlay_type != OVERLAY_TYPE_NONE) {
@@ -1431,6 +1502,8 @@ namespace FIFE {
 		rd.color[3] = a;
 		m_renderPrimitiveDatas.push_back(rd);
 
+		m_pIndices.push_back(m_pIndices.empty() ? 0 : m_pIndices.back() + 1);
+
 		RenderObject ro(GL_POINTS, 1);
 		m_renderObjects.push_back(ro);
 
@@ -1439,22 +1512,138 @@ namespace FIFE {
 
 	void RenderBackendOpenGL::drawLine(const Point& p1, const Point& p2, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
 		renderDataP rd;
-		rd.vertex[0] = static_cast<float>(p1.x)-0.375;
-		rd.vertex[1] = static_cast<float>(p1.y)-0.375;
+		rd.vertex[0] = static_cast<float>(p1.x)+0.375;
+		rd.vertex[1] = static_cast<float>(p1.y)+0.375;
 		rd.color[0] = r;
 		rd.color[1] = g;
 		rd.color[2] = b;
 		rd.color[3] = a;
 		m_renderPrimitiveDatas.push_back(rd);
 
-		rd.vertex[0] = static_cast<float>(p2.x)-0.375;
-		rd.vertex[1] = static_cast<float>(p2.y)-0.375;
+		rd.vertex[0] = static_cast<float>(p2.x)+0.375;
+		rd.vertex[1] = static_cast<float>(p2.y)+0.375;
 		m_renderPrimitiveDatas.push_back(rd);
 
+		m_pIndices.push_back(m_pIndices.empty() ? 0 : m_pIndices.back() + 1);
+		m_pIndices.push_back(m_pIndices.back() + 1);
+
 		RenderObject ro(GL_LINES, 2);
 		m_renderObjects.push_back(ro);
 	}
 
+	void RenderBackendOpenGL::drawThickLine(const Point& p1, const Point& p2, uint8_t width, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
+		float xDiff = p2.x - p1.x;
+		float yDiff = p2.y - p1.y;
+		float halfW = static_cast<float>(width) / 2.0;
+		float angle = Mathf::ATan2(yDiff, xDiff) * (180.0 / Mathf::pi()) + 90.0;
+		if (angle < 0.0) {
+			angle += 360.0;
+		} else if (angle > 360.0) {
+			angle -= 360.0;
+		}
+		angle *= Mathf::pi() / 180.0;
+		float cornerX = halfW * Mathf::Cos(angle);
+		float cornerY = halfW * Mathf::Sin(angle);
+
+		renderDataP rd;
+		rd.vertex[0] = static_cast<float>(p1.x) + cornerX;
+		rd.vertex[1] = static_cast<float>(p1.y) + cornerY;
+		rd.color[0] = r;
+		rd.color[1] = g;
+		rd.color[2] = b;
+		rd.color[3] = a;
+		m_renderPrimitiveDatas.push_back(rd);
+		rd.vertex[0] = static_cast<float>(p2.x) + cornerX;
+		rd.vertex[1] = static_cast<float>(p2.y) + cornerY;
+		m_renderPrimitiveDatas.push_back(rd);
+		rd.vertex[0] = static_cast<float>(p2.x) - cornerX;
+		rd.vertex[1] = static_cast<float>(p2.y) - cornerY;
+		m_renderPrimitiveDatas.push_back(rd);
+		rd.vertex[0] = static_cast<float>(p1.x) - cornerX;
+		rd.vertex[1] = static_cast<float>(p1.y) - cornerY;
+		m_renderPrimitiveDatas.push_back(rd);
+
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t indices[] = { index, index + 1, index + 2, index, index + 2, index + 3 };
+		m_pIndices.insert(m_pIndices.end(), indices, indices + 6);
+
+		RenderObject ro(GL_TRIANGLES, 6);
+		m_renderObjects.push_back(ro);
+	}
+
+	void RenderBackendOpenGL::drawPolyLine(const std::vector<Point>& points, uint8_t width, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
+		if (points.size() < 2) {
+			return;
+		}
+		std::vector<Point>::const_iterator it = points.begin();
+		if (width > 1) {
+			Point old = *it;
+			++it;
+			for (; it != points.end(); ++it) {
+				drawThickLine(old, *it, width, r, g, b, a);
+				drawFillCircle(old, width / 2, r, g, b, a);
+				old = *it;
+			}
+			drawFillCircle(old, width / 2, r, g, b, a);
+		} else {
+			renderDataP rd;
+			rd.color[0] = r;
+			rd.color[1] = g;
+			rd.color[2] = b;
+			rd.color[3] = a;
+			for (; it != points.end(); ++it) {
+				rd.vertex[0] = static_cast<float>((*it).x);
+				rd.vertex[1] = static_cast<float>((*it).y);
+				m_renderPrimitiveDatas.push_back(rd);
+				m_pIndices.push_back(m_pIndices.empty() ? 0 : m_pIndices.back() + 1);
+			}
+			RenderObject ro(GL_LINE_STRIP, points.size());
+			m_renderObjects.push_back(ro);
+		}
+	}
+
+	void RenderBackendOpenGL::drawBezier(const std::vector<Point>& points, int32_t steps, uint8_t width, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
+		if (points.size() < 2) {
+			return;
+		}
+		int32_t elements = points.size();
+		if (elements < 3 || steps < 2) {
+			return;
+		}
+
+		bool thick = width > 1;
+		float step = 1.0 / static_cast<float>(steps-1);
+		float t = 0.0;
+		Point old = getBezierPoint(points, elements+1, t);
+		if (thick) {
+			for (int32_t i = 0; i <= (elements*steps); ++i) {
+				t += step;
+				Point next = getBezierPoint(points, elements, t);
+				drawThickLine(old, next, width, r, g, b, a);
+				drawFillCircle(old, width / 2, r, g, b, a);
+				old = next;
+			}
+			drawFillCircle(old, width / 2, r, g, b, a);
+		} else {
+			renderDataP rd;
+			rd.color[0] = r;
+			rd.color[1] = g;
+			rd.color[2] = b;
+			rd.color[3] = a;
+			for (int32_t i = 0; i <= (elements*steps); ++i) {
+				t += step;
+				Point next = getBezierPoint(points, elements, t);
+				rd.vertex[0] = static_cast<float>(old.x);
+				rd.vertex[1] = static_cast<float>(old.y);
+				m_renderPrimitiveDatas.push_back(rd);
+				old = next;
+				m_pIndices.push_back(m_pIndices.empty() ? 0 : m_pIndices.back() + 1);
+			}
+			RenderObject ro(GL_LINE_STRIP, (elements*steps)+1);
+			m_renderObjects.push_back(ro);
+		}
+	}
+
 	void RenderBackendOpenGL::drawTriangle(const Point& p1, const Point& p2, const Point& p3, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
 		renderDataP rd;
 		rd.vertex[0] = static_cast<float>(p1.x);
@@ -1473,6 +1662,10 @@ namespace FIFE {
 		rd.vertex[1] = static_cast<float>(p3.y);
 		m_renderPrimitiveDatas.push_back(rd);
 
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t indices[] = { index, index + 1, index + 2 };
+		m_pIndices.insert(m_pIndices.end(), indices, indices + 3);
+
 		RenderObject ro(GL_TRIANGLES, 3);
 		m_renderObjects.push_back(ro);
 	}
@@ -1496,6 +1689,10 @@ namespace FIFE {
 		rd.vertex[0] = static_cast<float>(p.x);
 		m_renderPrimitiveDatas.push_back(rd);
 
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t indices[] = { index, index + 1, index + 2, index + 3 };
+		m_pIndices.insert(m_pIndices.end(), indices, indices + 4);
+
 		RenderObject ro(GL_LINE_LOOP, 4);
 		m_renderObjects.push_back(ro);
 	}
@@ -1519,7 +1716,11 @@ namespace FIFE {
 		rd.vertex[1] = static_cast<float>(p.y);
 		m_renderPrimitiveDatas.push_back(rd);
 
-		RenderObject ro(GL_QUADS, 4);
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t indices[] = { index, index + 1, index + 2, index, index + 2, index + 3 };
+		m_pIndices.insert(m_pIndices.end(), indices, indices + 6);
+
+		RenderObject ro(GL_TRIANGLES, 6);
 		m_renderObjects.push_back(ro);
 	}
 
@@ -1545,7 +1746,11 @@ namespace FIFE {
 		rd.vertex[1] = static_cast<float>(p4.y);
 		m_renderPrimitiveDatas.push_back(rd);
 
-		RenderObject ro(GL_QUADS, 4);
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t indices[] = { index, index + 1, index + 2, index, index + 2, index + 3 };
+		m_pIndices.insert(m_pIndices.end(), indices, indices + 6);
+
+		RenderObject ro(GL_TRIANGLES, 6);
 		m_renderObjects.push_back(ro);
 	}
 
@@ -1568,23 +1773,157 @@ namespace FIFE {
 		rd.vertex[0] = static_cast<float>(p.x-size);
 		m_renderPrimitiveDatas.push_back(rd);
 
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t indices[] = { index, index + 1, index + 2, index + 3 };
+		m_pIndices.insert(m_pIndices.end(), indices, indices + 4);
+
 		RenderObject ro(GL_LINE_LOOP, 4);
 		m_renderObjects.push_back(ro);
 	}
 
+	void RenderBackendOpenGL::drawCircle(const Point& p, uint32_t radius, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
+		// set side length to 5 and calculate needed divisions
+		int32_t subdivisions = round(Mathf::pi() / ( 5.0 / (2.0 * radius)));
+		if (subdivisions < 12) {
+			subdivisions = 12;
+		}
+		const float step = Mathf::twoPi()/subdivisions;
+		float angle = 0;
+
+		renderDataP rd;
+		rd.color[0] = r;
+		rd.color[1] = g;
+		rd.color[2] = b;
+		rd.color[3] = a;
+		for (uint16_t i = 0; i < subdivisions-1; ++i) {
+			rd.vertex[0] = radius * Mathf::Cos(angle) + p.x;
+			rd.vertex[1] = radius * Mathf::Sin(angle) + p.y;
+			m_renderPrimitiveDatas.push_back(rd);
+			angle += step;
+			m_pIndices.push_back(m_pIndices.empty() ? 0 : m_pIndices.back() + 1);
+		}
+
+		RenderObject ro(GL_LINE_LOOP, subdivisions-1);
+		m_renderObjects.push_back(ro);
+	}
+
+	void RenderBackendOpenGL::drawFillCircle(const Point& p, uint32_t radius, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
+		// set side length to 5 and calculate needed divisions
+		int32_t subdivisions = round(Mathf::pi() / ( 5.0 / (2.0 * radius)));
+		if (subdivisions < 12) {
+			subdivisions = 12;
+		}
+		const float step = Mathf::twoPi()/subdivisions;
+		float angle = Mathf::twoPi();
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t lastIndex = index;
+
+		// center vertex
+		renderDataP rd;
+		rd.vertex[0] = static_cast<float>(p.x);
+		rd.vertex[1] = static_cast<float>(p.y);
+		rd.color[0] = r;
+		rd.color[1] = g;
+		rd.color[2] = b;
+		rd.color[3] = a;
+		m_renderPrimitiveDatas.push_back(rd);
+		// reversed because of culling faces
+		for (uint16_t i = 0; i <= subdivisions; ++i) {
+			rd.vertex[0] = radius * Mathf::Cos(angle) + p.x;
+			rd.vertex[1] = radius * Mathf::Sin(angle) + p.y;
+			m_renderPrimitiveDatas.push_back(rd);
+			angle -= step;
+			// forms triangle with start index, the last and a new one
+			uint32_t indices[] = { index, lastIndex, ++lastIndex };
+			m_pIndices.insert(m_pIndices.end(), indices, indices + 3);
+		}
+		RenderObject ro(GL_TRIANGLES, (subdivisions+1) * 3);
+		m_renderObjects.push_back(ro);
+	}
+
+	void RenderBackendOpenGL::drawCircleSegment(const Point& p, uint32_t radius, int32_t sangle, int32_t eangle, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
+		const float step = Mathf::twoPi()/360;
+		int32_t elements = 0;
+		int32_t s = (sangle + 360) % 360;
+		int32_t e = (eangle + 360) % 360;
+		if (e == 0) {
+			e = 360;
+		}
+		if (s == e) {
+			return;
+		}
+
+		renderDataP rd;
+		rd.color[0] = r;
+		rd.color[1] = g;
+		rd.color[2] = b;
+		rd.color[3] = a;
+		float angle = static_cast<float>(s) * step;
+		for (;s <= e; ++s, angle += step, ++elements) {
+			rd.vertex[0] = radius * Mathf::Cos(angle) + p.x;
+			rd.vertex[1] = radius * Mathf::Sin(angle) + p.y;
+			m_renderPrimitiveDatas.push_back(rd);
+			m_pIndices.push_back(m_pIndices.empty() ? 0 : m_pIndices.back() + 1);
+		}
+
+		RenderObject ro(GL_LINE_STRIP, elements);
+		m_renderObjects.push_back(ro);
+	}
+
+	void RenderBackendOpenGL::drawFillCircleSegment(const Point& p, uint32_t radius, int32_t sangle, int32_t eangle, uint8_t r, uint8_t g, uint8_t b, uint8_t a) {
+		const float step = Mathf::twoPi() / 360;
+		int32_t s = (sangle + 360) % 360;
+		int32_t e = (eangle + 360) % 360;
+		if (e == 0) {
+			e = 360;
+		}
+		if (s == e) {
+			return;
+		}
+			
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t lastIndex = index;
+		// center vertex
+		renderDataP rd;
+		rd.vertex[0] = static_cast<float>(p.x);
+		rd.vertex[1] = static_cast<float>(p.y);
+		rd.color[0] = r;
+		rd.color[1] = g;
+		rd.color[2] = b;
+		rd.color[3] = a;
+		m_renderPrimitiveDatas.push_back(rd);
+		int32_t elements = 0;
+		// reversed because of culling faces
+		float angle = static_cast<float>(e) * step;
+		for (;s <= e; ++s, angle -= step, ++elements) {
+			rd.vertex[0] = radius * Mathf::Cos(angle) + p.x;
+			rd.vertex[1] = radius * Mathf::Sin(angle) + p.y;
+
+			m_renderPrimitiveDatas.push_back(rd);
+			// forms triangle with start index, the last and a new one
+			uint32_t indices[] = { index, lastIndex, ++lastIndex };
+			m_pIndices.insert(m_pIndices.end(), indices, indices + 3);
+		}
+
+		RenderObject ro(GL_TRIANGLES, elements * 3);
+		m_renderObjects.push_back(ro);
+	}
+
 	void RenderBackendOpenGL::drawLightPrimitive(const Point& p, uint8_t intensity, float radius, int32_t subdivisions, float xstretch, float ystretch, uint8_t red, uint8_t green, uint8_t blue) {
 		const float step = Mathf::twoPi()/subdivisions;
+		uint32_t elements = 0;
+		uint32_t index = m_pIndices.empty() ? 0 : m_pIndices.back() + 1;
+		uint32_t lastIndex = index;
+		// center vertex
 		renderDataP rd;
-		RenderObject ro(GL_TRIANGLES, 3);
-		for(float angle=0; angle<=Mathf::twoPi(); angle+=step){
-			rd.vertex[0] = static_cast<float>(p.x);
-			rd.vertex[1] = static_cast<float>(p.y);
-			rd.color[0] = red;
-			rd.color[1] = green;
-			rd.color[2] = blue;
-			rd.color[3] = intensity;
-			m_renderPrimitiveDatas.push_back(rd);
-
+		rd.vertex[0] = static_cast<float>(p.x);
+		rd.vertex[1] = static_cast<float>(p.y);
+		rd.color[0] = red;
+		rd.color[1] = green;
+		rd.color[2] = blue;
+		rd.color[3] = intensity;
+		m_renderPrimitiveDatas.push_back(rd);
+		for (float angle = 0; angle <= Mathf::twoPi(); angle += step, elements += 3) {
 			rd.vertex[0] = radius*Mathf::Cos(angle+step)*xstretch + p.x;
 			rd.vertex[1] = radius*Mathf::Sin(angle+step)*ystretch + p.y;
 			rd.color[0] = 0;
@@ -1596,13 +1935,16 @@ namespace FIFE {
 			rd.vertex[0] = radius*Mathf::Cos(angle)*xstretch + p.x;
 			rd.vertex[1] = radius*Mathf::Sin(angle)*ystretch + p.y;
 			m_renderPrimitiveDatas.push_back(rd);
-
-			m_renderObjects.push_back(ro);
+			// forms triangle with start index and two new ones
+			uint32_t indices[] = { index, ++lastIndex, ++lastIndex };
+			m_pIndices.insert(m_pIndices.end(), indices, indices + 3);
 		}
+		RenderObject ro(GL_TRIANGLES, elements);
+		m_renderObjects.push_back(ro);
 	}
 
 	void RenderBackendOpenGL::addImageToArray(uint32_t id, const Rect& rect, float const* st, uint8_t alpha, uint8_t const* rgba) {
-		RenderObject ro(GL_QUADS, 4, id);
+		RenderObject ro(GL_TRIANGLES, 6, id);
 		
 		// texture quad without alpha
 		if (alpha == 255 && !rgba) {
@@ -1629,6 +1971,10 @@ namespace FIFE {
 			m_renderTextureDatas.push_back(rd);
 
 			ro.color = false;
+
+			uint32_t index = m_tIndices.empty() ? 0 : m_tIndices.back() + 1;
+			uint32_t indices[] = { index, index + 1, index + 2, index, index + 2, index + 3 };
+			m_tIndices.insert(m_tIndices.end(), indices, indices + 6);
 		} else {
 			if (rgba) {
 				renderData2TC rd;
@@ -1668,6 +2014,10 @@ namespace FIFE {
 				ro.rgba[1] = rgba[1];
 				ro.rgba[2] = rgba[2];
 				ro.rgba[3] = rgba[3];
+
+				uint32_t index = m_tc2Indices.empty() ? 0 : m_tc2Indices.back() + 1;
+				uint32_t indices[] = { index, index + 1, index + 2, index, index + 2, index + 3 };
+				m_tc2Indices.insert(m_tc2Indices.end(), indices, indices + 6);
 			// texture quad with alpha
 			} else {
 				renderDataTC rd;
@@ -1697,6 +2047,10 @@ namespace FIFE {
 				m_renderTextureColorDatas.push_back(rd);
 
 				ro.color = true;
+
+				uint32_t index = m_tcIndices.empty() ? 0 : m_tcIndices.back() + 1;
+				uint32_t indices[] = { index, index + 1, index + 2, index, index + 2, index + 3 };
+				m_tcIndices.insert(m_tcIndices.end(), indices, indices + 6);
 			}
 		}
 		m_renderObjects.push_back(ro);
@@ -1735,7 +2089,11 @@ namespace FIFE {
 			rd.texel2[1] = st2[1];
 			m_renderMultitextureDatas.push_back(rd);
 
-			RenderObject ro(GL_QUADS, 4, id1, id2);
+			uint32_t index = m_tc2Indices.empty() ? 0 : m_tc2Indices.back() + 1;
+			uint32_t indices[] = { index, index + 1, index + 2, index, index + 2, index + 3 };
+			m_tc2Indices.insert(m_tc2Indices.end(), indices, indices + 6);
+
+			RenderObject ro(GL_TRIANGLES, 6, id1, id2);
 			ro.overlay_type = OVERLAY_TYPE_TEXTURES_AND_FACTOR;
 			ro.rgba[0] = rgba[0];
 			ro.rgba[1] = rgba[1];
@@ -1866,7 +2224,7 @@ namespace FIFE {
 				rd.texel2[1] = 0.0;
 				m_renderMultitextureDatasZ.push_back(rd);
 
-				RenderObject ro(GL_QUADS, 4, id);
+				RenderObject ro(GL_TRIANGLES, 6, id);
 				ro.color = true;
 				ro.overlay_type = OVERLAY_TYPE_COLOR;
 				ro.rgba[0] = rgba[0];
@@ -1944,8 +2302,7 @@ namespace FIFE {
 			rd.texel2[1] = st2[1];
 			m_renderMultitextureDatasZ.push_back(rd);
 
-			//RenderObject ro(GL_QUADS, 4, id1);
-			RenderObject ro(GL_QUADS, 4, id1, id2);
+			RenderObject ro(GL_TRIANGLES, 6, id1, id2);
 			ro.overlay_type = OVERLAY_TYPE_TEXTURES_AND_FACTOR;
 			ro.rgba[0] = rgba[0];
 			ro.rgba[1] = rgba[1];
@@ -2260,7 +2617,7 @@ namespace FIFE {
 		}
 
 		// can we use fbo?
-		if (GLEE_EXT_framebuffer_object && m_useframebuffer) {
+		if (GLEW_EXT_framebuffer_object && m_useframebuffer) {
 			glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_fbo_id);
 			glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
 				GL_TEXTURE_2D, targetid, 0);
@@ -2283,7 +2640,7 @@ namespace FIFE {
 
 		if (m_target_discard) {
 			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-		} else if (!m_target_discard && (!GLEE_EXT_framebuffer_object || !m_useframebuffer)) {
+		} else if (!m_target_discard && (!GLEW_EXT_framebuffer_object || !m_useframebuffer)) {
 			// if we wanna just add something to render target, we need to first render previous contents
 			addImageToArray(targetid, m_img_target->getArea(),
 				static_cast<GLImage*>(m_img_target.get())->getTexCoords(), 255, 0);
@@ -2296,7 +2653,7 @@ namespace FIFE {
 		// flush down what we batched
 		renderVertexArrays();
 
-		if (GLEE_EXT_framebuffer_object && m_useframebuffer) {
+		if (GLEW_EXT_framebuffer_object && m_useframebuffer) {
 			glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
 		} else {
 			bindTexture(0, static_cast<GLImage*>(m_img_target.get())->getTexId());
