@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2017 by the FIFE team                              *
+ *   Copyright (C) 2005-2019 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -22,7 +22,6 @@
 // Standard C++ library includes
 
 // 3rd party library includes
-#include <boost/bind.hpp>
 
 // FIFE includes
 // These includes are split up in two parts, separated by one empty line
@@ -48,7 +47,6 @@
 
 #include "view/camera.h"
 #include "view/visual.h"
-#include "cellrenderer.h"
 #include "instancerenderer.h"
 
 
@@ -139,7 +137,7 @@ namespace FIFE {
 		}
 		// init timer
 		m_timer.setInterval(m_interval);
-		m_timer.setCallback(boost::bind(&InstanceRenderer::check, this));
+		m_timer.setCallback(std::bind(&InstanceRenderer::check, this));
 		// create delete listener
 		m_delete_listener = new InstanceRendererDeleteListener(this);
 	}
@@ -163,7 +161,7 @@ namespace FIFE {
 		}
 		// init timer
 		m_timer.setInterval(m_interval);
-		m_timer.setCallback(boost::bind(&InstanceRenderer::check, this));
+		m_timer.setCallback(std::bind(&InstanceRenderer::check, this));
 		// create delete listener
 		m_delete_listener = new InstanceRendererDeleteListener(this);
 	}
@@ -218,10 +216,6 @@ namespace FIFE {
 				}
 			}
 		}
-		// Fixme
-		CellRenderer* cr = dynamic_cast<CellRenderer*>(cam->getRenderer("CellRenderer"));
-		Layer* fow_layer = cr->getFowLayer();
-		const bool check_fow = (fow_layer == layer && cr->isEnabledFogOfWar());
 
 		RenderList::iterator instance_it = instances.begin();
 		for (;instance_it != instances.end(); ++instance_it) {
@@ -230,15 +224,6 @@ namespace FIFE {
 			RenderItem& vc = **instance_it;
 			float vertexZ = vc.vertexZ;
 
-			if (check_fow) {
-				Cell* cell = layer->getCellCache()->getCell(instance->getLocationRef().getLayerCoordinates());
-				if (cell) {
-					if (cell->getFoWType() != CELLV_REVEALED) {
-						continue;
-					}
-				}
-			}
-
 			if (m_area_layer) {
 				InstanceToAreas_t::iterator areas_it = m_instance_areas.begin();
 				for(;areas_it != m_instance_areas.end(); areas_it++) {
@@ -411,10 +396,6 @@ namespace FIFE {
 				}
 			}
 		}
-		// Fixme
-		CellRenderer* cr = dynamic_cast<CellRenderer*>(cam->getRenderer("CellRenderer"));
-		Layer* fow_layer = cr->getFowLayer();
-		bool check_fow = (fow_layer == layer && cr->isEnabledFogOfWar());
 
 		RenderList::iterator instance_it = instances.begin();
 		for (;instance_it != instances.end(); ++instance_it) {
@@ -422,15 +403,6 @@ namespace FIFE {
 			Instance* instance = (*instance_it)->instance;
 			RenderItem& vc = **instance_it;
 
-			if (check_fow) {
-				Cell* cell = layer->getCellCache()->getCell(instance->getLocationRef().getLayerCoordinates());
-				if (cell) {
-					if (cell->getFoWType() != CELLV_REVEALED) {
-						continue;
-					}
-				}
-			}
-
 			if(m_area_layer) {
 				InstanceToAreas_t::iterator areas_it = m_instance_areas.begin();
 				for(;areas_it != m_instance_areas.end(); areas_it++) {
